# Récupérer les DOIs de l'UGA depuis Zenodo
## 2023-11-29, Maxence Larrieu


"""
## todo
- recherche itératives par types : dataset, image, video, software, physical object, other OU bien regarder si on requeter aussi dans le champs type directment

## Documentation
* Liste des métadonnées obligatoires lors du dépôts (upload_type, sub_type, publication_date, titre, creators, ) https://developers.zenodo.org/#representation
* Recherche dans l'interfance : https://zenodo.org/search?q=creators.affiliation%3Agrenoble&q=type%3Adataset&l=list&p=1&s=10&sort=bestmatch
* Doc dev API champs de requêtes classsiques https://developers.zenodo.org/#records
* doc champs poussées pour la recherche https://help.zenodo.org/guides/search/
* typologie des dépôts possiblent : publication: Publication, poster: Poster, presentation: Presentation, Dataset: Dataset, image: Image, video: Video/Audio, software: Software, lesson: Lesson, physicalobject: Physical object, other: Other


### Identifier les dépôts : 
- utilisation du champs `creators.affiliation` (multi affiliation)
- v2 : chercher aussi au niveau des "contributeurs" : `contributors.affiliation` et au niveau auteur `contributors.orcid`
- v2 : passer par les ORCID des 'auteurs' `creator.orcid`
- v2 : rechercher UGA comme financeur `grants.funder.doi`

## Notes sur la récupération
- exemple résultats de requete : https://zenodo.org/api/records?q=creators.affiliation%3A*grenoble*&type=dataset&page=6&size=100&sort=mostrecent&all_version=False&access_tpoken=4A1P6e4gIkvhpcNpVpI05C9yVCjJIDD3vljILoJ1wlyxT9VA70ZXyPMdjqHB)
- deux DOI identiques sont présent : un à la racine `[hits][doi]` et un autre dans `[hits][metadata][doi]`
- il y a des DOI renseignés par le déposant qui ne sont pas de Zenodo
"""

import pandas as pd, requests, json

parent_folder = "0-collect-data"
print("\n\nzenodo.py is launched")

## import API key. Recquired to req Zenodo API
with open(parent_folder + "/personnal-keys.json") as f : 
    ACCESS_TOKEN = json.load(f)["ZENODO_KEY"]

def req_zenodo(page_nb) :
    """
    retourne les jeux de données UGA depuis Zenodo
    page_nb précise le numéro de la page de la requête
    """

    r = requests.get(
        "https://zenodo.org/api/records",
        params = {
            "q" : "creators.affiliation:grenoble",
            "type" : "dataset",
            "page" : page_nb, 
            "size" : 100,
            "sort" : "mostrecent",
            "all_version" : False,
            "access_tpoken" : ACCESS_TOKEN
            } 
    )
    # to debug print(r.url)

    ## si pb présent dans les résultats
    if r.status_code != 200 : 
        return {
            "continue" : False,
            "content" : r
            }

    ## s'il n'y a pas de probleme
    res = r.json()
    
    return {
        "continue" : True,
        "content" : res["hits"]
        }


## var to initiate the loop
output_dois = []
current_page_number = 0
iterate = True

while iterate :
    
    current_page_number += 1
    print("new iteration page =", current_page_number)
    res = req_zenodo(current_page_number)

    # verifier si la requete a bien fonctionnée
    if not res["continue"] : 
        print("oups, problem with the query")
        print(res["content"].status_code ) ## res[content] contient la réponse de la requête
        break

    # pour la premiere iteration, extraire le nb de DOI a récupérer
    if current_page_number == 1 : 
        nb_dois_to_get = res["content"]["total"]
        print("\tfirst iteration. DOIs to get =", nb_dois_to_get )

    ## parcourir tous les DOIs et les placer dans la liste
    ## placer le DOI uniquement si une donnée est présente
    [output_dois.append( item.get("doi") ) for item in res["content"]["hits"] if item.get("doi")]

    ## managing the loop
    ### if I have all the DOIs inside my outputs, then I had finish
    if len(output_dois) >= nb_dois_to_get : 
        iterate = False


## ____z____ exporter la liste de DOI au format txt

with open(parent_folder + "/zenodo-dois.txt", 'w') as f :
    [f.write(f"{line}\n") for line in output_dois]

